#region ... Copyright  2005 Roland Weigelt ...
/*
This software is provided 'as-is', without any expressed or
implied warranty. In no event will the author be held liable
for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you
	use this software in a product, an acknowledgment in the
	product documentation is NOT required.
	
2.	Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
	
3.	This notice may not be removed or altered from any source
	or altered source distribution.

Copyright  2005 Roland Weigelt (linkbox@roland-weigelt.de)
*/
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace Weigelt.GhostDoc.Gui
{
	/// <summary>
	/// Shows one or more lines, each containing a link and/or (surrounding) text.
	/// </summary>
	/// <remarks>
	/// This is not a robust, bullet-proof, commercial-quality, general-purpose control,
	/// it's more of a base-class for application-specific controls.
	/// </remarks>
	public class LinkBox : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Panel c_pnlLinks;
		private int m_nItemCount=0;
		private System.ComponentModel.Container components = null; // Required designer variable.

		#region ... Component Designer generated code ...
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.c_pnlLinks = new System.Windows.Forms.Panel();
			this.SuspendLayout();
			// 
			// c_pnlLinks
			// 
			this.c_pnlLinks.AutoScroll = true;
			this.c_pnlLinks.BackColor = System.Drawing.SystemColors.Window;
			this.c_pnlLinks.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.c_pnlLinks.Dock = System.Windows.Forms.DockStyle.Fill;
			this.c_pnlLinks.DockPadding.All = 2;
			this.c_pnlLinks.Location = new System.Drawing.Point(0, 0);
			this.c_pnlLinks.Name = "c_pnlLinks";
			this.c_pnlLinks.Size = new System.Drawing.Size(150, 150);
			this.c_pnlLinks.TabIndex = 0;
			// 
			// LinkBox
			// 
			this.Controls.Add(this.c_pnlLinks);
			this.Name = "LinkBox";
			this.ResumeLayout(false);

		}
		#endregion

		#region ... Protected hooks and overrides ...

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Called when a link is clicked, raises the <see cref="LinkClicked"/> event.
		/// </summary>
		/// <param name="args"></param>
		protected virtual void OnLinkClicked(LinkBoxEventArgs args)
		{
			if (this.LinkClicked!=null)
				this.LinkClicked(this, args);
		}

		#endregion

		#region ... Private helper methods ...

		/// <summary>
		/// Handles click events of the <see cref="LinkLabel"/>
		/// controls used internally.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e">Event arguments</param>
		private void HandleLinkLabelLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			LinkLabel lnkSource=(LinkLabel) sender;
			LinkBoxEventArgs objArgs=new LinkBoxEventArgs(lnkSource.TabIndex, lnkSource.Tag);
			OnLinkClicked(objArgs);
		}

		/// <summary>
		/// Adds a new <see cref="LinkLabel"/> control to the panel, behind
		/// the existing link labels.
		/// </summary>
		/// <param name="fixedText">The fixed part of the text, containing "{0}" where the link text should be inserted.</param>
		/// <param name="linkText">The text for the link to be inserted at the position of the "{0}" in the fixed text.</param>
		/// <param name="tag">Object containing data about the link.</param>
		/// <returns></returns>
		private LinkLabel AppendLinkLabel(string fixedText, string linkText, object tag)
		{
			LinkLabel lnkItem=new LinkLabel();
			lnkItem.AutoSize = true;
			lnkItem.Dock=DockStyle.Top;
			lnkItem.LinkArea = new System.Windows.Forms.LinkArea(0,0);
			lnkItem.Location = new System.Drawing.Point(0, 0); // no need to determine the actual position
			lnkItem.Name = "lnkItem"+m_nItemCount.ToString();
			lnkItem.TabIndex = m_nItemCount;
			lnkItem.TabStop = true;
			lnkItem.Tag=tag;
			SetText(lnkItem, fixedText, linkText);
			lnkItem.LinkClicked+=new LinkLabelLinkClickedEventHandler(HandleLinkLabelLinkClicked);

			c_pnlLinks.Controls.Add(lnkItem);
			c_pnlLinks.Controls.SetChildIndex(lnkItem, 0);

			//
			// After we have added the item, the layout is updated, so we can
			// use the layout properties.
			//
			int nNewAutoScrollWidth=(c_pnlLinks.AutoScrollMinSize.Width<lnkItem.Width)?lnkItem.Width:c_pnlLinks.AutoScrollMinSize.Width;
			int nNewAutoScrollHeight=lnkItem.Top+lnkItem.Height;
			c_pnlLinks.AutoScrollMinSize=new Size(nNewAutoScrollWidth, nNewAutoScrollHeight);
			++m_nItemCount;
			return lnkItem;
		}

		/// <summary>
		/// Gets the <see cref="LinkLabel"/> control representing the the line with
		/// the specified index.
		/// </summary>
		/// <param name="index">Index of the line (zero-based)</param>
		/// <returns></returns>
		private LinkLabel GetLinkLabel(int index)
		{
			if (index>=m_nItemCount)
				throw new ArgumentOutOfRangeException("index");
			//
			// Because of the docking for the LinkLabel controls, the link labels are
			// ordered opposite to the index of the line they represent. Thus the control
			// at index 0 of the control collection represents the last line.
			//
			return (LinkLabel) c_pnlLinks.Controls[c_pnlLinks.Controls.Count-1-index];
		}

		/// <summary>
		/// Sets the text for the specified <see cref="LinkLabel"/> control.
		/// </summary>
		/// <param name="linkLabel">The control to set the text on.</param>
		/// <param name="fixedText">The fixed part of the text, containing "{0}" where the link text should be inserted.</param>
		/// <param name="linkText">The text for the link to be inserted at the position of the "{0}" in the fixed text.</param>
		private void SetText(LinkLabel linkLabel, string fixedText, string linkText)
		{
			int nLinkAreaStart=fixedText.IndexOf("{0}");
			if (nLinkAreaStart==-1)
			{
				//
				// No insertion point specified, so we'll ignore the linkText parameter
				//
				linkLabel.Text=fixedText;
				linkLabel.LinkArea=new LinkArea(0,0);
				return;
			}

			linkLabel.Text=String.Format(fixedText, linkText);
			linkLabel.LinkArea=new LinkArea(nLinkAreaStart, linkText.Length);
		}

		#endregion

		//===================================================
		// Public
		//===================================================

		/// <summary>
		/// Raised when a link was clicked.
		/// </summary>
		public event LinkBoxEventHandler LinkClicked;

		//###################################################
		//
		// Note: the following methods are currently public so you can play around
		// with the control in the demo project. If you use LinkBox purely as a base
		// class for other controls, you could/should make them protected.
		//
		//###################################################

		/// <summary>
		/// Adds the specified text.
		/// </summary>
		/// <param name="text"></param>
		public void AddText(string text)
		{
			this.AppendLinkLabel(text, null, null);
		}

		/// <summary>
		/// Adds a line consisting of the specified fixed text, with a link inserted
		/// at the position of the "{0}" in the fixed text.
		/// </summary>
		/// <param name="fixedText">The fixed part of the text, containing "{0}" where the link text should be inserted.</param>
		/// <param name="linkText">The text for the link to be inserted at the position of the "{0}" in the fixed text.</param>
		public void AddLink(string fixedText, string linkText)
		{
			this.AppendLinkLabel(fixedText, linkText, null);
		}

		/// <summary>
		/// Adds a line consisting of the specified fixed text, with a link inserted
		/// at the position of the "{0}" in the fixed text.
		/// </summary>
		/// <param name="fixedText">The fixed part of the text, containing "{0}" where the link text should be inserted.</param>
		/// <param name="linkText">The text for the link to be inserted at the position of the "{0}" in the fixed text.</param>
		/// <param name="tag">Object containing data about the link.</param>
		public void AddLink(string fixedText, string linkText, object tag)
		{
			this.AppendLinkLabel(fixedText, linkText, tag);
		}

		/// <summary>
		/// Updates the auto scroll size of the panel containing the link controls.
		/// </summary>
		public void UpdateAutoScrollSize()
		{
			int nNewAutoScrollWidth=0;
			int nNewAutoScrollHeight=0;
			int nHeight;

			foreach (Control ctl in c_pnlLinks.Controls)
			{
				if (ctl.Width>nNewAutoScrollWidth)
					nNewAutoScrollWidth=ctl.Width;
				nHeight=ctl.Top+ctl.Height;
				if (nHeight>nNewAutoScrollHeight)
					nNewAutoScrollHeight=nHeight;
			}
			c_pnlLinks.AutoScrollMinSize=new Size(nNewAutoScrollWidth, nNewAutoScrollHeight);
		}

		/// <summary>
		/// Sets the text of the line with the specified index.
		/// </summary>
		/// <param name="index">Index of the line (zero-based)</param>
		/// <param name="text">The text for the line.</param>
		public void SetText(int index, string text)
		{
			SetText(GetLinkLabel(index), text, null);
		}

		/// <summary>
		/// Sets the text of the line with the specified index.
		/// </summary>
		/// <param name="index">Index of the line (zero-based)</param>
		/// <param name="fixedText">The fixed part of the text, containing "{0}" where the link text should be inserted.</param>
		/// <param name="linkText">The text for the link to be inserted at the position of the "{0}" in the fixed text.</param>
		public void SetText(int index, string fixedText, string linkText)
		{
			SetText(GetLinkLabel(index), fixedText, linkText);
		}

		/// <summary>
		/// Gets the number of items.
		/// </summary>
		public int ItemCount
		{
			get { return m_nItemCount; }
		}

		#region ... Constructor ...

		/// <summary>
		/// Creates a new <see cref="LinkBox"/> control.
		/// </summary>
		public LinkBox()
		{
			InitializeComponent();
		}

		#endregion
	}
}
